package org.openxma.dsl.generator.helper;

import org.eclipse.emf.ecore.EObject;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.ControlEventMapping;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.EnterEventMapping;
import org.openxma.dsl.pom.model.EventFunction;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.GuiElementEventMapping;
import org.openxma.dsl.pom.model.IGuiElementWithEvent;
import org.openxma.dsl.pom.model.InitEventMapping;
import org.openxma.dsl.pom.model.LeaveEventMapping;
import org.openxma.dsl.pom.model.PredefinedCommand;
import org.openxma.dsl.pom.naming.PomNameProvider;

import at.spardat.xma.guidesign.DataLabel;
import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAComposite;
import at.spardat.xma.guidesign.XMALabel;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAPagingControl;
import at.spardat.xma.guidesign.XMATable;
import at.spardat.xma.guidesign.XMAText;
import at.spardat.xma.guidesign.XMATree;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.FlexFactory;
import at.spardat.xma.guidesign.flex.ISelectFunctionCaller;
import at.spardat.xma.guidesign.flex.MenuItem;
import at.spardat.xma.guidesign.flex.XMAFunction;

public class EventExtension {
    
    private static final String COMMAND_PROPERTY_NAME = "command";    

    static class LifeCycleEventMappings {                       

        private InitEventMapping initEventMapping ;
        private EnterEventMapping enterEventMapping;
        private LeaveEventMapping leaveEventMapping;

        LifeCycleEventMappings(EventMappingList eventMappingList) {
            for (EventMapping eventMapping : eventMappingList.getMapping()) {
                if (eventMapping instanceof InitEventMapping && initEventMapping == null) {
                    initEventMapping = (InitEventMapping)eventMapping;
                    continue;
                }
                if (eventMapping instanceof EnterEventMapping && enterEventMapping == null) {
                    enterEventMapping = (EnterEventMapping)eventMapping;
                    continue;
                }
                if (eventMapping instanceof LeaveEventMapping && leaveEventMapping == null) {
                    leaveEventMapping = (LeaveEventMapping)eventMapping;
                    continue;
                }
            }
        }
    }
    
    public static void setCommandProperty(XMAFunction xmaFunction, Command command) {
        EObjectPropertiesAdapter.setProperty(xmaFunction,COMMAND_PROPERTY_NAME,command);               
    }
    
    public static Command getCommandProperty(XMAFunction xmaFunction) {
        Object value = EObjectPropertiesAdapter.getProperty(xmaFunction,COMMAND_PROPERTY_NAME);
        if (value instanceof Command || value == null) {
            return (Command)value;
        }
        throw new RuntimeException("The property '"+COMMAND_PROPERTY_NAME+"' is not of type 'org.openxma.xmadsl.model.Command'");
    }    

    public static XMAWidget getWidgetForGuiElementWithEvent(XMAPage xmaPage, IGuiElementWithEvent element) {
        String name = PomNameProvider.XMA_ID.apply(element);
        return Util.getWidgetForName(xmaPage,name);
    }

    private static XMAFunction getXMAFunctionForEventFunction(EventFunction eventFunction, XMAComponent xmaComponent, XMAPage xmaPage) {
        XMAFunction xmaFunction = null;
        if (eventFunction.getCommand() != null) {
            Command command = eventFunction.getCommand();
            xmaFunction = Util.getXMAFunctionForName(xmaComponent,xmaPage,command.getName());
        } else {
            if (xmaPage == null) {
                throw new IllegalArgumentException("The xmaPage must not be null in case the eventFunction is a predefined command");
            }
            PredefinedCommand pc = eventFunction.getPredefinedCommand();
            String name = getNameOfPredefinedCommand(pc);
            xmaFunction = Util.getXMAFunctionForName(xmaComponent,xmaPage,name);
            if (xmaFunction == null) {
                xmaFunction = FlexFactory.eINSTANCE.createXMAFunction();
                xmaFunction.setNamFunction(name);
                xmaFunction.setYnServerSide(false);
                xmaPage.getFunctions().add(xmaFunction);
            }
        }
        return xmaFunction;
    }
    
    private static String getNameOfPredefinedCommand(PredefinedCommand pc) {
        switch(pc.getValue()) {
        case PredefinedCommand.CLOSE_OK_VALUE: return "closeOK";
        case PredefinedCommand.CLOSE_CANCEL_VALUE: return "closeCancel";
        default: throw new RuntimeException("Unknown value for predefined command");
        }
    }

    public static void extendEventMappingForComponent(EventMappingList eventMappingList, XMAComponent xmaComponent) {
        if (eventMappingList == null) {
            throw new IllegalArgumentException("The eventMappingList must not be null");
        }

        LifeCycleEventMappings em = new LifeCycleEventMappings(eventMappingList);
        if (em.initEventMapping != null) {
            for (EventFunction eventFunction : em.initEventMapping.getEventFunctions()) {
                XMAFunction xmaFunction = getXMAFunctionForEventFunction(eventFunction,xmaComponent,null);
                xmaComponent.getInitFunction().add(xmaFunction);
            }
        }
        if (em.enterEventMapping != null) {
            for (EventFunction eventFunction : em.enterEventMapping.getEventFunctions()) {
                XMAFunction xmaFunction = getXMAFunctionForEventFunction(eventFunction,xmaComponent,null);
                xmaComponent.getEnterFunction().add(xmaFunction);
            }
        }
        if (em.leaveEventMapping != null) {
            for (EventFunction eventFunction : em.leaveEventMapping.getEventFunctions()) {
                XMAFunction xmaFunction = getXMAFunctionForEventFunction(eventFunction,xmaComponent,null);
                xmaComponent.getLeaveFunction().add(xmaFunction);
            }
        }
    }

    public static void createInitEventMapping(EventMappingList eventMappingList, XMAComponent xmaComponent, XMAPage xmaPage) {
        if (eventMappingList == null) {
            throw new IllegalArgumentException("The eventMappingList must not be null");
        }
        LifeCycleEventMappings em = new LifeCycleEventMappings(eventMappingList);
        if (em.initEventMapping != null) {
            for (EventFunction eventFunction : em.initEventMapping.getEventFunctions()) {
                XMAFunction xmaFunction = getXMAFunctionForEventFunction(eventFunction,xmaComponent,xmaPage);
                xmaPage.getInitFunction().add(xmaFunction);
            }
        }
        if (em.enterEventMapping != null) {
            for (EventFunction eventFunction : em.enterEventMapping.getEventFunctions()) {
                XMAFunction xmaFunction = getXMAFunctionForEventFunction(eventFunction,xmaComponent,xmaPage);
                xmaPage.getEnterFunction().add(xmaFunction);
            }
        }
        if (em.leaveEventMapping != null) {
            for (EventFunction eventFunction : em.leaveEventMapping.getEventFunctions()) {
                XMAFunction xmaFunction = getXMAFunctionForEventFunction(eventFunction,xmaComponent,xmaPage);
                xmaPage.getLeaveFunction().add(xmaFunction);
            }
        }
    }

    public static void createWidgetEventMapping(XMAWidget xmaWidget, ControlEventMapping controlEventMapping, XMAComponent xmaComponent, XMAPage xmaPage) {
        for (EventFunction eventFunction : controlEventMapping.getEventFunctions()) {
            XMAFunction xmaFunction = getXMAFunctionForEventFunction(eventFunction,xmaComponent,xmaPage);
            setEventFunction(xmaWidget,controlEventMapping.getEvent(),xmaFunction);
        }
    }
    
    public static void createMenuEventMapping(GuiElementEventMapping guiElemEventMapping, XMAComponent xmaComponent, XMAPage xmaPage) {
        for (EventFunction eventFunction : guiElemEventMapping.getEventFunctions()) {
            XMAFunction xmaFunction = getXMAFunctionForEventFunction(eventFunction,xmaComponent,xmaPage);
            MenuItem menuItem = Util.getMenuItemForName(xmaPage,PomNameProvider.INSTANCE.getFullyQualifiedName(guiElemEventMapping.getControl()).getLastSegment());
            menuItem.setXMAFunction(xmaFunction);
        }
    }



    public static XMAPagingControl getPagingControlForXMATable(XMATable xmaTable) {
        XMAComposite composite = (XMAComposite)xmaTable.eContainer();
        for (EObject obj : composite.eContents()) {
            if (obj instanceof XMAPagingControl) {
                return (XMAPagingControl)obj;
            }
        }
        return null;
    }

    public static XMAWidget setEventFunction(XMAWidget w, EventType ev,    XMAFunction xmaFunction) {
        if (w instanceof XMALabel || w instanceof DataLabel || w instanceof XMAText) return w; // Event mapping is done in Strategy
        switch (ev.getValue()) {
        case EventType.ON_CLICK_VALUE:
        case EventType.ON_SELECTION_VALUE:
            Util.castAndSetSelectFunction(w, xmaFunction);
            break;
        case EventType.ON_DOUBLE_CLICK_VALUE:
        case EventType.ON_DEFAULT_SELECTION_VALUE:
            Util.castAndSetDefSelectFunction(w, xmaFunction);
            break;
        case EventType.ON_EXPAND_VALUE:
            ((XMATree) w).getExpandFunction().add(xmaFunction);
            break;
        case EventType.ON_COLLAPSE_VALUE:
            ((XMATree) w).getCollapseFunction().add(xmaFunction);
            break;
        case EventType.ON_LOAD_VALUE:
            XMATable xmaTable = (XMATable)w;
            XMAPagingControl pc = getPagingControlForXMATable(xmaTable);
            if (pc != null) {
                pc.getNavigateFunction().add(xmaFunction);
            }
            break;
        default:
            if (w instanceof XMATable) {
                Util.castAndSetDefSelectFunction(w, xmaFunction);
            } else if (w instanceof ISelectFunctionCaller) {
                Util.castAndSetSelectFunction(w, xmaFunction);
            } else {
                throw new RuntimeException(
                        "Eventmapping without explicitly defined or default function: "
                                + w.getNamInstance() + " -> "
                                + xmaFunction.getNamFunction());
            }
        }
        return w;
    }
}

